<?php

namespace Zen\Payment;

/**
 * Class Util
 *
 * @package Payment
 */
class Util
{

    const METHOD_REQUEST_POST = 'POST';
    const ENVIRONMENT_PRODUCTION = 'production';
    const ENVIRONMENT_SANDBOX = 'sandbox';
    const ENVIRONMENT_LOCAL = 'local';

    /**
     * @var array
     */
    private static $hashMethods = [
        'sha224' => 'sha224',
        'sha256' => 'sha256',
        'sha384' => 'sha384',
        'sha512' => 'sha512',
    ];

    /**
     * @var array
     */
    private static $transactionStatuses = [
        'CANCELED' => 'CANCELED',
        'AUTHORIZED' => 'AUTHORIZED',
        'PENDING' => 'PENDING',
        'REJECTED' => 'REJECTED',
        'ACCEPTED' => 'ACCEPTED',
    ];

    /**
     * @param string $merchantTransactionId
     *
     * @return string
     */
    public static function extractMerchantTransactionId($merchantTransactionId)
    {

        return explode('#', $merchantTransactionId)[0];
    }

    /**
     * @return array
     */
    public static function getCurrencies()
    {
        return include 'Helpers/Currencies.php';
    }

    /**
     * @param string $variable
     *
     * @return bool
     */
    public static function isJson($variable)
    {
        json_decode($variable);

        return (json_last_error() === JSON_ERROR_NONE);
    }

    /**
     * @param string $hashMethod
     * @param string $data
     * @param string $serviceKey
     *
     * @return string
     */

    public static function hashSignature($hashMethod, $data, $serviceKey)
    {

        return hash($hashMethod, $data . $serviceKey);
    }

    /**
     * @param string $hashMethod
     *
     * @return string
     */
    public static function getHashMethod($hashMethod)
    {

        if (isset(self::$hashMethods[$hashMethod])) {
            return self::$hashMethods[$hashMethod];
        }

        return '';
    }

    /**
     * @return array
     */
    public static function getTransactionStatuses()
    {
        return self::$transactionStatuses;
    }

    /**
     * @param float $amount
     *
     * @return int
     */
    public static function convertAmountToFractional($amount)
    {
        return (int)self::multiplyValues(round($amount, 2), 100, 0);
    }

    /**
     * @param number $firstValue
     * @param number $secondValue
     * @param number $precision
     *
     * @return float
     */
    public static function multiplyValues($firstValue, $secondValue, $precision)
    {
        return round($firstValue * $secondValue, $precision);
    }

    /**
     * @param int $amount
     *
     * @return float
     */
    public static function convertAmountToMain($amount)
    {

        return (float)round($amount / 100, 2);
    }

    /**
     * @param string $orderId
     *
     * @return string
     */
    public static function generateMerchantTransactionId($orderId)
    {
        return sprintf('%04x%04x-%04x-%04x-%04x-%04x%04x%04x',
                // 32 bits for "time_low"
                mt_rand(0, 0xffff), mt_rand(0, 0xffff),

                // 16 bits for "time_mid"
                mt_rand(0, 0xffff),

                // 16 bits for "time_hi_and_version",
                // four most significant bits holds version number 4
                mt_rand(0, 0x0fff) | 0x4000,

                // 16 bits, 8 bits for "clk_seq_hi_res",
                // 8 bits for "clk_seq_low",
                // two most significant bits holds zero and one for variant DCE1.1
                mt_rand(0, 0x3fff) | 0x8000,

                // 48 bits for "node"
                mt_rand(0, 0xffff), mt_rand(0, 0xffff), mt_rand(0, 0xffff)
            ) . '&orderId=' . $orderId;
    }

}
