<?php
/**
 * Zen
 *
 * @author    Zen
 * @copyright Copyright (c) 2021 Zen
 * @license   https://opensource.org/licenses/AFL-3.0  Academic Free License ("AFL") v. 3.0
 *
 * https://www.zen.com/
 */

if (!defined('_PS_VERSION_')) {
    exit;
}

include_once(_PS_MODULE_DIR_ . 'zencom/libraries/payment-core/vendor/autoload.php');


/**
 * Class Zen
 *
 * @property string tab
 * @property array ps_versions_compliancy
 * @property int need_instance
 * @property string author
 * @property string name
 * @property string version
 * @property string displayName
 * @property string description
 * @property string confirm_uninstall
 */
class Zencom extends PaymentModule
{

    /**
     * Sets the Information for the Module manager
     * Also creates an instance of this class
     */
    public function __construct()
    {
        $this->name = 'zencom';
        $this->tab = 'payments_gateways';
        $this->version = '1.0.9';
        $this->author = 'Zen';
        $this->need_instance = 1;
        $this->ps_versions_compliancy = [
            'min' => '1.6',
            'max' => '1.6',
        ];
        $this->module_key = 'ffc7f50d8b9c2970c145fb6ac764d24a';

        parent::__construct();

        $this->displayName = $this->l('Zen');
        $this->description = $this->l('Visa, MasterCard, BLIK ect.');
        $this->confirm_uninstall = $this->l('Are you sure you want to uninstall Zen module?');
    }

    /**
     * This function installs the Zen Module
     *
     * @return boolean
     */
    public function install()
    {

        if (!(
            parent::install()
            && $this->registerHook('payment')
            && $this->registerHook('displayPaymentReturn')
            && $this->registerHook('displayBackOfficeHeader')
            && $this->registerHook('displayHeader')
            && Configuration::updateValue('ZEN_ENABLED', 0)
            && Configuration::updateValue('ZEN_TERMINAL_ID', '')
            && Configuration::updateValue('ZEN_PAYWALL_SECRET', '')
            && Configuration::updateValue('ZEN_IPN_SECRET', '')
            && Configuration::updateValue('ZEN_API_KEY', '')
            && Configuration::updateValue('ZEN_PAYMENT_TITLE', '')
        )) {
            return false;
        }

        if (Validate::isInt(Configuration::get('PAYMENT_ZEN_NEW_STATUS'))
            xor (
            Validate::isLoadedObject(new OrderState(Configuration::get('PAYMENT_ZEN_NEW_STATUS'))))
        ) {
            $orderStateNew = new OrderState();
            $missingLang = true;

            $langs = [
                'en' => 'Payment Zen: awaiting for confirmation',
                'pl' => 'Płatność Zen: oczekuje na potwierdzenie',
            ];

            foreach ($langs as $lang => $message) {
                $langId = Language::getIdByIso($lang);
                if (isset($langId)) {
                    $orderStateNew->name[$langId] = $message;
                    $missingLang = false;
                }
            }

            if ($missingLang) {
                $langId = (int)$this->context->language->id;
                $orderStateNew->name[$langId] = $langs['en'];
            }

            $orderStateNew->send_email = false;
            $orderStateNew->invoice = false;
            $orderStateNew->unremovable = false;
            $orderStateNew->color = "black";

            if (!$orderStateNew->add()) {
                $this->_errors[] = $this->l('There was an Error installing the module. Cannot add new order state.');

                return false;
            }
            if (!Configuration::updateValue('PAYMENT_ZEN_NEW_STATUS', $orderStateNew->id)) {
                $this->_errors[] = $this->l('There was an Error installing the module. Cannot update new order state.');

                return false;
            }

            copy(
                _PS_MODULE_DIR_ . $this->name . '/views/img/zen-icon.png',
                _PS_IMG_DIR_ . 'os/' . $orderStateNew->id . '.png'
            );
        }

        return true;
    }

    /**
     * This function uninstalls the Zen Module
     *
     * @return boolean
     */
    public function uninstall()
    {
        return Configuration::deleteByName('ZEN_ENABLED')
            && Configuration::deleteByName('ZEN_TERMINAL_ID')
            && Configuration::deleteByName('ZEN_PAYMENT_TITLE')
            && Configuration::deleteByName('ZEN_PAYWALL_SECRET')
            && Configuration::deleteByName('ZEN_IPN_SECRET')
            && Configuration::deleteByName('ZEN_API_KEY')
            && parent::uninstall();
    }

    /**
     * Display configuration form.
     *
     * @return mixed
     */
    public function getContent()
    {

        $msg = '';

        if (Tools::isSubmit('submitZen')) {
            if ($this->saveConfiguration()) {
                $msg = [
                    'type' => 'success',
                    'message' => $this->l('Configuration updated successfully'),
                ];
            } else {
                $msg = [
                    'type' => 'error',
                    'message' => $this->l('There was an error saving your configuration'),
                ];
            }
        }

        $this->context->smarty->assign([
            'zen_form' =>
                './index.php?tab=AdminModules&configure=zen&token='
                . Tools::getAdminTokenLite('AdminModules')
                . '&tab_module='
                . $this->tab
                . '&module_name=zen',
            'zen_enabled' => Configuration::get('ZEN_ENABLED'),
            'zen_terminal_id' => Configuration::get('ZEN_TERMINAL_ID'),
            'zen_paywall_secret' => Configuration::get('ZEN_PAYWALL_SECRET'),
            'zen_ipn_secret' => Configuration::get('ZEN_IPN_SECRET'),
            'zen_api_key' => Configuration::get('ZEN_API_KEY'),
            'zen_msg' => $msg,
            'zen_payment_title' => Configuration::get('ZEN_PAYMENT_TITLE'),
            'zen_payment_title_default' => $this->getPaymentTitleDefault(),
            'zen_all_currencies' => \Zen\Payment\Util::getCurrencies(),
            // region messages
            'zen_msg_configuration' => $this->l('Configuration'),
            'zen_msg_hint_installation' => $this->l('The module requires a configuration in the 
            zen administration panel. Log in to the administration panel, go to stores tab and 
            enter the notification address in the appropriate field and copy the configuration 
            keys. Copy the keys into the fields described below.'),
            'zen_msg_pay_with_zen' => $this->l('Pay with Zen'),
            'zen_msg_terminal_uuid' => $this->l('Terminal UUID'),
            'zen_msg_paywall_secret' => $this->l('Paywall Secret'),
            'zen_msg_ipn_secret' => $this->l('IPN Secret'),
            'zen_msg_api_key' => $this->l('Terminal API Key'),
            'zen_msg_payment_title' => $this->l('Payment title'),
            'zen_msg_currencies' => $this->l('Currencies'),
            'zen_msg_notification_url' => $this->l('Your address for notifications'),
            'zen_msg_on' => $this->l('On'),
            'zen_msg_off' => $this->l('Off'),
            'zen_msg_save' => $this->l('Save'),
            // endregion
        ]);

        return $this->fetchTemplate('admin.tpl');
    }

    /**
     * @return bool
     */
    private function saveConfiguration()
    {

        Configuration::updateValue('ZEN_ENABLED', pSQL(Tools::getValue('zen_enabled')));
        Configuration::updateValue('ZEN_TERMINAL_ID', pSQL(Tools::getValue('zen_terminal_id')));
        Configuration::updateValue('ZEN_PAYWALL_SECRET', pSQL(Tools::getValue('zen_paywall_secret')));
        Configuration::updateValue('ZEN_IPN_SECRET', pSQL(Tools::getValue('zen_ipn_secret')));
        Configuration::updateValue('ZEN_API_KEY', pSQL(Tools::getValue('zen_api_key')));
        Configuration::updateValue('ZEN_PAYMENT_TITLE', pSQL(Tools::getValue('zen_payment_title')));

        return true;
    }

    /**
     * @return string
     */
    private function getPaymentTitleDefault()
    {
        return 'Cards, Bank transfers, BLIK, paysafecard, PayPal with';
    }

    /**
     * @param string $name
     *
     * @return mixed
     */
    public function fetchTemplate($name)
    {
        $views = 'views/templates/';
        if (file_exists(dirname(__FILE__) . '/' . $name)) {
            return $this->display(__FILE__, $name);
        }

        if (file_exists(dirname(__FILE__) . '/' . $views . 'hook/' . $name)) {
            return $this->display(__FILE__, $views . 'hook/' . $name);
        }

        if (file_exists(dirname(__FILE__) . '/' . $views . 'front/' . $name)) {
            return $this->display(__FILE__, $views . 'front/' . $name);
        }

        if (file_exists(dirname(__FILE__) . '/' . $views . 'admin/' . $name)) {
            return $this->display(__FILE__, $views . 'admin/' . $name);
        }

        return $this->display(__FILE__, $name);
    }

    /**
     * @param $name
     * @param $type
     *
     * @return string
     */
    public static function buildTemplatePath($name, $type)
    {
        return 'module:zencom/views/templates/' . $type . '/' . $name . '.tpl';
    }

    /**
     * @param bool $http
     * @param bool $entities
     *
     * @return string
     */
    public function getModuleAddress($http = true, $entities = true)
    {
        return $this->getShopDomainAddress($http, $entities) . (__PS_BASE_URI__ . 'modules/' . $this->name . '/');
    }

    /**
     * @param bool $http
     * @param bool $entities
     *
     * @return string
     */
    public static function getShopDomainAddress($http = false, $entities = false)
    {
        if (method_exists('Tools', 'getShopDomainSsl')) {
            return Tools::getShopDomainSsl($http, $entities);
        }

        if (!($domain = Configuration::get('PS_SHOP_DOMAIN_SSL'))) {
            $domain = Tools::getHttpHost();
        }

        if ($entities) {
            $domain = htmlspecialchars($domain, ENT_COMPAT, 'UTF-8');
        }

        if ($http) {
            $domain = (Configuration::get('PS_SSL_ENABLED')
                    ? 'https://'
                    : 'http://') . $domain;
        }

        return $domain;
    }

    /**
     * Hook that run on mobile version of Prestashop.
     */
    public function hookDisplayMobileHeader()
    {
        $this->hookHeader();
    }

    /**
     * Hook that adds css and js in admin panel. For newer version of Prestashop.
     */
    public function hookDisplayBackOfficeHeader()
    {
        $this->context->controller->addCSS($this->_path . 'views/css/zen-admin.css', 'all');
        $this->context->controller->addCSS($this->_path . 'views/css/zen-front.css', 'all');
    }

    /**
     * @param array $params
     *
     * @return bool|mixed
     */
    public function hookPayment($params)
    {

        if (!Configuration::get('ZEN_ENABLED')
            || !Configuration::get('ZEN_TERMINAL_ID')
            || !Configuration::get('ZEN_PAYWALL_SECRET')
            || !Configuration::get('ZEN_IPN_SECRET')
            || !Configuration::get('ZEN_API_KEY')) {
            return false;
        }

        $this->smarty->assign([
            'zen_ps_version' => _PS_VERSION_,
            'zen_img' => $this->getLogoPath(),
            'zen_payment_url' => $this->context->link->getModuleLink('zencom', 'payment'),
            'zen_payment_title' => Configuration::get('ZEN_PAYMENT_TITLE') ?: $this->getPaymentTitleDefault(),
        ]);

        return $this->fetchTemplate('payment.tpl');
    }

    /**
     * @param string $file
     *
     * @return mixed
     */
    public function getLogoPath($file = 'logo.svg')
    {
        return Media::getMediaPath(_PS_MODULE_DIR_ . $this->name . '/views/img/' . $file);
    }

    /** Display order info in confirmation page */
    public function hookDisplayPaymentReturn($params)
    {
        if (Tools::version_compare(_PS_VERSION_, '1.7', '>') || $this->active == false) {
            return false;
        }

        $id_order = $params['objOrder']->id;
        $order = new Order($id_order);

        if ($order->getCurrentOrderState()->id != Configuration::get('PS_OS_ERROR')) {
            $this->smarty->assign('status', 'ok');
        }

        $carrier = new Carrier((int)$order->id_carrier, (int)$order->id_lang);

        if ($order->total_discounts > 0) {
            $this->context->smarty->assign('total_old', (float)$order->total_paid - $order->total_discounts);
        }

        $products = $order->getProducts();

        $this->smarty->assign(array(
            'id_order' => $order->id,
            'products' => $products,
            'carrier' => $carrier->name,
            'currency' => $params['currencyObj']->sign,
            'reference' => $order->reference,
            'total' => Tools::displayPrice($params['total_to_pay'], $params['currencyObj'], false),
        ));

        return $this->fetchTemplate('confirmation.tpl');
    }
}
